# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/2.1_repr.codeberta.ipynb (unless otherwise specified).

__all__ = []

# Cell
# Imports
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

from annoy import AnnoyIndex
from mpl_toolkits.mplot3d import Axes3D
from sklearn import decomposition
from pathlib import Path
from transformers import pipeline
from tqdm.notebook import tqdm